package com.tzld.longarticle.recommend.server.service.exterior.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.common.enums.GhTypeEnum;
import com.tzld.longarticle.recommend.server.common.response.CommonResponse;
import com.tzld.longarticle.recommend.server.mapper.growth.GhDetailMapper;
import com.tzld.longarticle.recommend.server.model.vo.GhDetailVo;
import com.tzld.longarticle.recommend.server.repository.model.GhDetail;
import com.tzld.longarticle.recommend.server.repository.model.GhDetailExample;
import com.tzld.longarticle.recommend.server.service.exterior.GhDetailService;
import com.tzld.longarticle.recommend.server.util.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
public class GhDetailServiceImpl implements GhDetailService {

    @Autowired
    private GhDetailMapper ghDetailMapper;

    @Override
    public CommonResponse<Page<GhDetailVo>> getGhDetailList(Integer pageNum, Integer pageSize, String ghId) {
        if (pageNum == null) {
            pageNum = 1;
        }
        if (pageSize == null) {
            pageSize = 20;
        }
        if (pageSize > 100) {
            pageSize = 100;
        }
        Page<GhDetailVo> page = new Page<>();
        page.setCurrentPage(pageNum);
        page.setPageSize(pageSize);
        GhDetailExample example = new GhDetailExample();
        if (StringUtils.isNotEmpty(ghId)) {
            example.createCriteria().andGhIdEqualTo(ghId);
        }
        example.setPage(page);
        long total = ghDetailMapper.countByExample(example);
        int totalSize = (int) (total % pageSize == 0 ? (total / pageSize) : (total / pageSize + 1));
        page.setTotalSize(totalSize);
        List<GhDetail> ghDetails = ghDetailMapper.selectByExample(example);
        List<GhDetailVo> ghDetailVos = new ArrayList<>();
        if (!CollectionUtils.isEmpty(ghDetails)) {
            for (GhDetail ghDetail : ghDetails) {
                GhDetailVo ghDetailVo = new GhDetailVo();
                BeanUtils.copyProperties(ghDetail, ghDetailVo);
                ghDetailVo.setTypeName(GhTypeEnum.getTypeName(ghDetailVo.getType()));
                ghDetailVos.add(ghDetailVo);
            }
        }
        page.setObjs(ghDetailVos);
        return CommonResponse.success(page);
    }

    @Override
    public CommonResponse<Void> addGhDetail(GhDetailVo ghDetailVo) {
        if (ghDetailVo == null
                || StringUtils.isEmpty(ghDetailVo.getGhId())
                || StringUtils.isEmpty(ghDetailVo.getGhName())
                || ghDetailVo.getType() == null
                || StringUtils.isEmpty(ghDetailVo.getCategory1())) {
            CommonResponse.create(500, "参数错误");
        }

        try {
            GhDetail ghDetail = new GhDetail();
            BeanUtils.copyProperties(ghDetailVo, ghDetail);
            ghDetailMapper.insert(ghDetail);
            return CommonResponse.success();
        } catch (Exception e) {
            log.error("addGhDetail error", e);
        }
        return CommonResponse.create(500, "插入失败，可能已经存在该记录");
    }

    @Override
    public CommonResponse<Void> updateDetail(GhDetailVo ghDetailVo) {
        try {
            GhDetail ghDetail = new GhDetail();
            BeanUtils.copyProperties(ghDetailVo, ghDetail);
            ghDetailMapper.updateByPrimaryKeySelective(ghDetail);
            return CommonResponse.success();
        } catch (Exception e) {
            log.error("updateDetail error", e);
        }
        return CommonResponse.create(500, "修改失败");
    }

    @Override
    public CommonResponse<Void> refreshGhDetail() {
        GhDetailExample example = new GhDetailExample();
        example.createCriteria().andTypeEqualTo(1);
        List<GhDetail> ghDetails = ghDetailMapper.selectByExample(example);
        if (CollectionUtils.isEmpty(ghDetails)) {
            return CommonResponse.success();
        }
        for (GhDetail ghDetail : ghDetails) {
            getGhDetail(ghDetail);
            log.info("getGhDetail = {}", ghDetail);
            ghDetailMapper.updateByPrimaryKeySelective(ghDetail);
        }
        return CommonResponse.success();
    }

    public void getGhDetail(GhDetail ghDetail) {
        String baseUrl = "http://aigc-api.cybertogether.net";
        WebClient webClient = WebClient.builder()
                .baseUrl(baseUrl)
                .build();
        Mono<String> mono = webClient.get()
                .uri(String.format("/aigc/publish/account/algorithm/wechat/list?gh_id=%s", ghDetail.getGhId()))
                .retrieve().bodyToMono(String.class);
        String block = mono.block();
        JSONArray jsonArray = JSONArray.parseArray(block);
        if (jsonArray != null && !jsonArray.isEmpty()) {
            JSONObject jsonObject = jsonArray.getJSONObject(0);
            ghDetail.setGhName(jsonObject.getString("name"));
        }
    }
}
