package com.tzld.longarticle.recommend.server.web;


import com.tzld.longarticle.recommend.server.common.enums.GhTypeEnum;
import com.tzld.longarticle.recommend.server.common.response.CommonResponse;
import com.tzld.longarticle.recommend.server.model.vo.GhDetailVo;
import com.tzld.longarticle.recommend.server.model.vo.GhTypeVo;
import com.tzld.longarticle.recommend.server.repository.model.GhDetail;
import com.tzld.longarticle.recommend.server.service.exterior.GhDetailService;
import com.tzld.longarticle.recommend.server.util.page.Page;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@RestController
@RequestMapping("/gh")
public class GhDetailController {

    @Autowired
    private GhDetailService ghDetailService;

    @GetMapping("/getList")
    public CommonResponse<Page<GhDetailVo>> getGhDetailList(@RequestParam Integer pageNum, @RequestParam Integer pageSize, @RequestParam String ghId) {
        return ghDetailService.getGhDetailList(pageNum, pageSize, ghId);
    }

    @PostMapping("/add")
    public CommonResponse<Void> addGhDetail(@RequestBody GhDetailVo detailVo) {
        return ghDetailService.addGhDetail(detailVo);
    }

    @PostMapping("/update")
    public CommonResponse<Void> updateGhDetail(@RequestBody GhDetailVo detailVo) {
        return ghDetailService.updateDetail(detailVo);
    }

    @GetMapping("/refresh")
    public CommonResponse<Void> refreshGhDetail() {
        return ghDetailService.refreshGhDetail();
    }

    @GetMapping("/types")
    public CommonResponse<List<GhTypeVo>> getAllGhTypes() {
        return CommonResponse.success(Arrays.stream(GhTypeEnum.values())
                .map(ghTypeEnum -> new GhTypeVo(ghTypeEnum.type, ghTypeEnum.name))
                .collect(Collectors.toList()));
    }
}
